package nckd.jimin.jyyy.hr.haos.staff.plugin.task;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import kd.bos.context.RequestContext;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.exception.KDException;
import kd.bos.logging.Log;
import kd.bos.logging.LogFactory;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.orm.util.CollectionUtils;
import kd.bos.schedule.executor.AbstractTask;
import kd.bos.util.StringUtils;
import kd.sdk.plugin.Plugin;
import kd.hr.hbp.business.servicehelper.HRBaseServiceHelper;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 2025-04-16 Tyx
 * 组织变动数据同步MDM
 * 1.组织名称变化/组织调整时，需要把该组织下所有岗位及人员也传给MDM
 * 2.其余变化时，只需要传递组织信息，岗位人员不需要
 */
public class SyncAdminOrgTask extends AbstractTask implements Plugin {

    private static final Log log = LogFactory.getLog(SyncAdminOrgTask.class);
    private static String changeOperDetail_entity = "haos_changeoperdetail";
    private static String adminOrgDetail_entity = "haos_adminorgdetail";
    private static String orgChgRecord_entity = "homs_orgchgrecord";
    private static String orgChgRecord_e_entity = "orgchgentry";
    private static String startTime = "";
    private static String endTime = "";



    @Override
    public void execute(RequestContext requestContext, Map<String, Object> map) throws KDException {
        /**
         * 获取调度作业下配置的参数：
         * isAuto : 如果配置为是，则取日志中上一次同步时间至今发生变化的组织信息，为否则优先取以下两个手工配置的时间参数
         * startTime : 手工配置开始时间 yyyy-MM-dd hh24:mi:ss
         * endTime : 手工配置结束时间 yyyy-MM-dd hh24:mi:ss
         */
        log.info("-------- SyncAdminOrgTask 开始执行同步行政组织 --------");
        //最后调MDM的jsonObject
        JSONObject ob = new JSONObject();
        initDateRange(map);
        HRBaseServiceHelper helper = new HRBaseServiceHelper(orgChgRecord_entity);
        String selectProperties = "adminorg,orgchgentry.changescene.number,createtime";
        //查出发生过变化的组织
        DynamicObjectCollection changeOperateCol = helper.queryOriginalCollection(selectProperties, this.getQFilters());
        //查询出来为空直接返回
        if(CollectionUtils.isEmpty(changeOperateCol)) {
            log.info("-------- 未查询到变化组织 -------- ");
            SyncUtil.createLog(SyncUtil.v_error, startTime, endTime, "未查询到变化组织", "未调用", SyncUtil.SyncOrg);
        }
        else {
            //发生变化的组织Map
            //按照变动场景编码分成不同的List，后续根据场景分开处理
            //ToDo 如果查询时间内，某个组织先发生组织调整，再修改除名称外其他信息怎么处理？
            //按照组织分组，只取时间段内最新的一条记录
            //暂时只处理（10，20，30，40，50）;
            //ToDo（80，90）目前前台界面做不了该业务，后续考虑;
            //ToDo 60尚不明确，需确认；
            //1010_S	    组织新设
            //1020_S	    上级调整
            //1030_S	    组织更名
            //1040_S	    组织停用
            //1050_S	    初始化新增
            //1060_S	    初始化禁用
            //1080_S	    组织合并
            //1090_S	    组织拆分
            //CS_1110_SY01	组织修订
//            Map<String, List<DynamicObject>> changeOperateMap = (Map)changeOperateCol.stream().collect(Collectors.groupingBy(
//                            dyx -> dyx.getString("orgchgentry.changescene.number")
//                    ));
            //按组织ID汇总
            Map<Long, DynamicObject> changeOperateMap = (Map)changeOperateCol.stream().collect(Collectors.toMap((dyx) -> {
                return dyx.getLong("adminorg");
            }, (dyx) -> {
                return dyx;
            }, (key1, key2) -> {
                return key2;
            }));
            log.info("-------- 变化组织数（不去重）：" + changeOperateCol.size() + " --------");
            log.info("-------- 变化组织数（去重）：" + changeOperateMap.keySet().size() + " --------");
            log.info("-------- 变化组织Id：" + changeOperateMap.keySet() + " --------");

            // ToDo 根据变动操作字段获取发生了组织调整的组织，后续需要根据这些组织同步下级岗位和人员
            //组织历史查询，如果查询时间内发生多次变化，
            //haos_adminorgdetail 组织历史查询 所属表t_haos_adminorg Id与t_org_org一样
            selectProperties = "id,boid,number,name,orglongname,level,nckd_easid,parentorg.id,parentorg.name,parentorg.number,establishmentdate,bsed,enable,adminorgtype.name,disabledate,modifytime";
            QFilter idQFilter = new QFilter("id", "in", changeOperateMap.keySet());
            DynamicObject[] orgDyArr = new HRBaseServiceHelper(adminOrgDetail_entity).query(selectProperties, new QFilter[]{idQFilter});
            //处理下长编码/长名称，星瀚内是4VTX1ACRU8A9!4VU/45/K1PJZ!4W5LG9NNW9EW这种格式 -- 直接取行政组织结构里面的长编码
            Map<Long, DynamicObject> bosOrgMap = SyncUtil.initOrgInfo();
            log.info("-------- 查询完组织详细信息数：" + orgDyArr.length + " --------");
            //构建入参
            buildJSON(ob, orgDyArr, changeOperateMap, bosOrgMap);
            //获取调用接口地址
            String url = SyncUtil.getUrl("org_url");
            log.info("-------- 组织同步调用url : " + url + " --------");
            //调用接口
            JSONObject response = SyncUtil.doPostByHttpClient(url, ob);
            //处理返回结果
            String status = SyncUtil.dealResponseStatus(response);
            //记录日志
            SyncUtil.createLog(status, startTime, endTime, ob.toJSONString(), response.toJSONString(), SyncUtil.SyncOrg);
        }
    }

    /**
     * 构建最后调用MDM接口的参数
     * @param ob
     */
    public void buildJSON(JSONObject ob, DynamicObject[] orgDyArr, Map<Long, DynamicObject> changeOperateMap, Map<Long, DynamicObject> bosOrgMap) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        JSONArray obj = new JSONArray();
        for(DynamicObject org : orgDyArr) {
            JSONObject data = new JSONObject(true);
            //主体标签 默认JY
            data.put("MAIN_BODY_ID", "JY");
            //主键 EAS老组织用EASID 否则用星瀚ID
            String primary_key = StringUtils.isBlank(org.getString("nckd_easid")) ? org.getPkValue().toString() : org.getString("nckd_easid");
            data.put("ADM_PRIMARY_KEY", primary_key);
            //长编码
            data.put("ADM_LONG_CODE", bosOrgMap.get(org.getLong("id")).getString("structure.longnumber"));
            //组织名称
            data.put("MD_DESCRIPTION", org.getString("name"));
            //上级行政组织ID 看是否有easid 有就传EASID 否则传星瀚ID
            data.put("ADM_PARENT_NUMBER", org.getString("parentorg.id"));
            //上级组织名称
            data.put("ADM_PARENT_NAME", org.getString("parentorg.name"));
            //行政组织长名称 orglongname没值 去平台那边取
            data.put("ADM_LONG_NAME", bosOrgMap.get(org.getLong("id")).getString("structure.fullname"));
            //业务板块 默认11-济煜医药
            data.put("ADM_BUSINESS_SECTOR", "11");
            //级别
            data.put("MD_LEVEL", org.getInt("level"));
            //行政组织类型
            data.put("ADM_TYPE", org.getString("adminorgtype.name"));
            //行政组织旧编码，s-HR中的组织导到星瀚中前面加了JY-，后续新增的使用星瀚的编码规则
            String orgNumber = org.getString("number");
            if(orgNumber.indexOf("JY-") == 0) {
                data.put("ADM_OLD_NUMER", orgNumber.substring(3, orgNumber.length()));
            }
            else {
                data.put("ADM_OLD_NUMER", orgNumber);
            }
            //创建时间 取行政组织的创建时间
            Date createDate = bosOrgMap.get(org.getLong("id")).getDate("createtime");
            data.put("ADM_CREATION_TIME", sdf.format(createDate));
            //最后修改时间
            data.put("ADM_LAST_UPTIME", sdf.format(org.getDate("modifytime")));
            //行政组织是否封存 enable 0-已停用 1-已启用
            int enable = org.getInt("enable");
            if(enable == 0) {
                data.put("ADM_SEALUP", "是");
                //行政组织封存时间
                Date disableDate = org.getDate("disabledate");
                data.put("ADM_STORAGE_TIME", sdf.format(disableDate));
            }
            else {
                data.put("ADM_SEALUP", "否");
                data.put("ADM_STORAGE_TIME", null);
            }

            //是否成本中心实体组织 TODO
            data.put("ADM_ENTITY_COST_CENTER", null);
            //是否成本中心组织 TODO
            data.put("ADM_COST_CENTER", null);
            //成本中心是否封存 TODO
            data.put("ADM_COST_CENTER_SEALUP", null);
            obj.add(data);
        }
        SyncUtil.setJSONArraySorted(obj, "MD_LEVEL");
        ob.put("obj", obj);
    }


    /**
     * 根据调度作业上配置的参数初始化时间范围
     * @param map
     */
    public void initDateRange(Map<String, Object> map) {
        boolean isAuto = Boolean.valueOf(map.get("isAuto").toString());
        if(!isAuto) {
            startTime = map.get("startTime").toString();
            endTime = map.get("endTime").toString();
        }
        //取最近一次日志的时间
        else {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            // ToDo
            //startTime = "";
            endTime = sdf.format(new Date());
        }
    }

    /**
     * 构建查询变化组织的过滤条件
     * @return
     */
    public QFilter[] getQFilters() {
        try {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            QFilter filter = new QFilter("createtime", ">=", sdf.parse(startTime));
            filter.and("createtime", "<", sdf.parse(endTime));
            //filter.and("org.otclassify", "=", 1010L);
            //filter.and("adminorg.number", QCP.equals, "JY-20240612-0009");
            log.info("-------- 查询组织变化过滤条件：" + filter.toString() + " --------");
            return new QFilter[]{filter};
        } catch (ParseException e) {
            throw new RuntimeException(e);
        }
    }

}